﻿// Author: Orlys
// Github: https://github.com/Orlys

namespace Yuuna.Contracts.Plugins
{
    using System;
    using System.Collections.Immutable;

    using Yuuna.Contracts.Optimization;
    using Yuuna.Contracts.Patterns;
    using Yuuna.Contracts.Semantics;
    using Yuuna.Contracts.TextSegmention;
    
    public abstract class PluginBase
    {
        private readonly PatternFactory _patternfactory;
        private StrategyBase _strategy;

        internal IPatternSet Patterns => this._patternfactory;

        public PluginBase()
        {
            this._patternfactory = new PatternFactory(this);
        }

        [Obsolete]
        internal IImmutableList<IScore> Evaluate(IImmutableList<string> cutted)
        {
            return this._strategy.Assess(this, cutted, this._patternfactory);
        }
        [Obsolete]
        internal Invoke SelectBest(IScore score)
        {
            return this._patternfactory.TryGet(score.Pattern, out var b) ? b : null;
        }

        internal bool Initialize(StrategyBase strategy, ITextSegmenter textSegmenter, IGroupManager groupManager)
        {
            this._strategy = strategy;

            try
            {
                this.BuildPatterns(groupManager, this._patternfactory);
                textSegmenter.Load(groupManager);
                this.AfterInitialize();
                return true;
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                return false;
            }
        }


        /// <summary>
        /// 在初始化後引發。
        /// </summary>
        protected virtual void AfterInitialize()
        {
        }

        /// <summary>
        /// 建立模式規則。
        /// </summary>
        /// <param name="g"></param>
        /// <param name="p"></param>
        protected abstract void BuildPatterns(IGroupManager g, IPatternBuilder p);
    }
}